%Matlab code for figure 5 in section 3.2.2 in "Low-cost, readily available 3D microscopy imaging system with variable focus spinner"
%The original code was written in C++, but the implementing environment was case/pc by case/pc. 
%Matlab code with almost the same process method was shown here. 
%Code was checked under windows/mac in Matlab R2016a.

clear all; close all; clc;

% load images
NoImage=12;%Number of images 
NoStart=0; %Filename number start
%this path should be changed according to the image folder
startfn='images/chip/raw/'; 

ImgArr=cell(1,NoImage); %save raw images for warp processed
ImgArrLap=cell(1,NoImage);%save raw images for Lap processed
ImgArrBW=cell(1,NoImage);%save images after Lap processed

for i=1:NoImage
    disp(strcat('Loading Image No',num2str(i),'/',num2str(NoImage)));
    ImgArr{i}=imread(strcat(startfn,num2str(NoStart-1+i),'.bmp')); % save raw images
    ImgArrLap{i}=ImgArr{i};% copy another raw images for Lap
end

%Image brightnesses adjustment 
%the brightness of each image was made the same, giving the pixels in the final fused all-in-focus image uniform brightness
%such as due to transmission of 3 mm plate was ~92%
b1 = mean2(rgb2gray(ImgArr{1})); %ref images
for i=2:NoImage
    b2 = mean2(rgb2gray(ImgArr{i}));
    ImgArr{i} = ImgArr{i} + (b1-b2);
end

%operators
%Size in pixels to search for edges 

%build laplace (instead of using inbuilt 'fspecial')
OSize_L=8;  %size oflaplace kernels
Mid_L=ceil(OSize_L/2);
c=1;
Laplace=zeros(OSize_L,OSize_L);
for i=1:OSize_L
    for j=1:OSize_L
        x=i-Mid_L;
        y=j-Mid_L;
        Laplace(i,j)=(-1/pi/c/c/c/c)*(1-(x*x+y*y)/2/c/c)*exp(-(x*x+y*y)/2/c/c);
    end
end
%build gauss (instead of using inbuilt 'fspecial')
OSize_G=11;  %size of gausskernels
Mid_G=ceil(OSize_G/2);
c=2.7;
Gauss  =zeros(OSize_G,OSize_G);
for i=1:OSize_G
    for j=1:OSize_G
        x=i-Mid_G;
        y=j-Mid_G;
        Gauss(i,j) = exp(-(((x*x)/2/c/c+(y*y)/2/c/c)));
    end
end

%edge detect all images
for i=1:NoImage
    disp(strcat('Edge Detecting Image No',num2str(i),'/',num2str(NoImage)));
    
    Temp = conv2(double(rgb2gray(ImgArrLap{i})),Gauss,'same');
    ImgArrBW{i} = conv2(Temp,Laplace,'same');
    
    SizeS = size(ImgArrBW{i});
    ImgArrBW{i}(1:OSize_L,:) = 0;%to aviod no NG white frame around the images
    ImgArrBW{i}(:,1:OSize_L) = 0;
    ImgArrBW{i}(SizeS(1)-OSize_L-1:SizeS(1),:) = 0;
    ImgArrBW{i}(:,SizeS(2)-OSize_L-1:SizeS(2)) = 0;
        
    ImgArrBW{i} = abs(ImgArrBW{i});
    
    disp(strcat('Dilating Image No ',num2str(i),'/',num2str(NoImage)));    
    se = strel('ball',3,3);
    for imdilate_i=1:5
    	ImgArrBW{i} = imdilate(ImgArrBW{i},se,'same');
    end
end

 %scale according to the focal length
 for i=1:NoImage
     index_re = (100+ 0.3408*(i-1))/100; %re-scaling factor according to the focal length change Eq(5) N_plate = 1.5170, N_air = 1, thus (1 - N_air/N_plate)
     
     tform = affine2d([1/index_re 0 0; 0 1/index_re 0; 0 0 1]);
     Rfixed = imref2d(size(ImgArr{1}));
     ImgArr{i} = imwarp(ImgArr{i},tform,'OutputView',Rfixed, 'FillValues',[0;0;0]);
     if i == 1 
         FillValue = 255;
     else
         FillValue = 0;
     end
     ImgArrBW{i} = imwarp(ImgArrBW{i},tform,'OutputView',Rfixed, 'FillValues',FillValue);
 end
 
 %phase alliment the image
ImgCorr_Ref = ImgArr{1};
for i=1:NoImage

    tformEstimate = imregcorr(rgb2gray(ImgArr{i}),rgb2gray(ImgCorr_Ref),'translation');% phase-only-coreection factor
    Rfixed = imref2d(size(ImgCorr_Ref));
    ImgArr{i} = imwarp(ImgArr{i},tformEstimate,'OutputView',Rfixed, 'FillValues',[0;0;0]);
     if i == 1 
         FillValue = 255;
     else
         FillValue = 0;
     end
    ImgArrBW{i} = imwarp(ImgArrBW{i},tformEstimate,'OutputView',Rfixed, 'FillValues',FillValue);

    ImgCorr_Ref = ImgArr{i};
end

%order stack
Stack = ones(SizeS);% all 1 array
Stack_smooth = zeros(SizeS,'double');% all 0 array
StackVal = zeros(SizeS);% all 0 array
for i=1:NoImage
    disp(strcat('Stacking Image No',num2str(i),'/',num2str(NoImage)));

    for j=1:SizeS(1)
        for k=1:SizeS(2)
            if (ImgArrBW{i}(j,k)>StackVal(j,k))%if ImgArrBW > 0 
                StackVal(j,k)=ImgArrBW{i}(j,k);%save the val
                Stack(j,k)=i;%save the index
            end
        end 
    end 
end

%Build smooth Depth map 
SizeI=size(ImgArr{1});
for j=1:SizeI(1)
    for k=1:SizeI(2)

        Index=double(Stack(j,k));

        if (Index<1)
            Index=1;
        end
        if (Index>NoImage)
            Index=NoImage;
        end
        
        if (Index == 1 || Index == NoImage)
             Stack_smooth(j,k) = Index;
        else
            peak = Index*1.0 + double(ImgArrBW{Index - 1}(j,k) - ImgArrBW{Index + 1}(j,k)) / double(2.0 * (ImgArrBW{Index - 1}(j,k) - 2.0 * ImgArrBW{Index}(j,k) + ImgArrBW{Index + 1}(j,k)));
            Stack_smooth(j,k) = peak;
        end
    end
end

% do 2d median filtering
Stack = medfilt2(Stack, [45 45]);
Stack_smooth = medfilt2(Stack_smooth, [45 45]);

%Combine into all-in-focus image
SharpImg=ImgArr{1};
for j=1:SizeI(1)
    for k=1:SizeI(2)

        Index=int8(Stack(j,k));

        if (Index<1)
            Index=1;
        end
        
        if (Index>NoImage)
            Index=NoImage;
        end

        SharpImg(j,k,:)=ImgArr{Index}(j,k,:);
    end
end

%show and save image
figure('Name','Raw')
for i=1:NoImage
    subplot(3,4,i);
    imshow(ImgArr{i},[]);hold on
end
figure('Name','Lap')
for i=1:NoImage
    subplot(3,4,i);
    imshow(ImgArrBW{i},[]);hold on
end
figure('Name','SharpImg');imshow(SharpImg,[]);
figure('Name','MapImg new');imshow(-Stack_smooth,[])